@@ -1,20 +1,31 @@
 /* visibility.c - Wrapper for all public functions
- *      Copyright (C) 2008 g10 Code GmbH
+ * Copyright (C) 2008, 2012 g10 Code GmbH
  *
  * This file is part of KSBA.
  *
  * KSBA is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of either
  *
- * KSBA is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *   - the GNU Lesser General Public License as published by the Free
+ *     Software Foundation; either version 3 of the License, or (at
+ *     your option) any later version.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * or
+ *
+ *   - the GNU General Public License as published by the Free
+ *     Software Foundation; either version 2 of the License, or (at
+ *     your option) any later version.
+ *
+ * or both in parallel, as here.
+ *
+ * KSBA is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
+ * License for more details.
+ *
+ * You should have received a copies of the GNU General Public License
+ * and the GNU Lesser General Public License along with this program;
+ * if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -23,6 +34,7 @@
 #define _KSBA_INCLUDED_BY_VISIBILITY_C
 #include "util.h"
 
+
 /*--version.c --*/
 const char *
 ksba_check_version (const char *req_version)
@@ -1226,3 +1238,82 @@ ksba_name_get_uri (ksba_name_t name, int idx)
 {
   return _ksba_name_get_uri (name, idx);
 }
+
+
+/*-- der-encoder.c --*/
+void
+ksba_der_release (ksba_der_t d)
+{
+  if (d)
+    _ksba_der_release (d);
+}
+
+ksba_der_t
+ksba_der_builder_new (unsigned int nitems)
+{
+  return _ksba_der_builder_new (nitems);
+}
+
+void
+ksba_der_builder_reset (ksba_der_t d)
+{
+  _ksba_der_builder_reset (d);
+}
+
+void
+ksba_der_add_ptr (ksba_der_t d, int cls, int tag,
+                  void *value, size_t valuelen)
+{
+  _ksba_der_add_ptr (d, cls, tag, value, valuelen);
+}
+
+void
+ksba_der_add_val (ksba_der_t d, int cls, int tag,
+                  const void *value, size_t valuelen)
+{
+  _ksba_der_add_val (d, cls, tag, value, valuelen);
+}
+
+void
+ksba_der_add_int (ksba_der_t d, const void *value, size_t valuelen,
+                  int force_positive)
+{
+  _ksba_der_add_int (d, value, valuelen, force_positive);
+}
+
+void
+ksba_der_add_oid (ksba_der_t d, const char *oidstr)
+{
+  _ksba_der_add_oid (d, oidstr);
+}
+
+void
+ksba_der_add_bts (ksba_der_t d, const void *value, size_t valuelen,
+                  unsigned int unusedbits)
+{
+  _ksba_der_add_bts (d, value, valuelen, unusedbits);
+}
+
+void
+ksba_der_add_der (ksba_der_t d, const void *der, size_t derlen)
+{
+  _ksba_der_add_der (d, der, derlen);
+}
+
+void
+ksba_der_add_tag (ksba_der_t d, int cls, int tag)
+{
+  _ksba_der_add_tag (d, cls, tag);
+}
+
+void
+ksba_der_add_end (ksba_der_t d)
+{
+  _ksba_der_add_end (d);
+}
+
+gpg_error_t
+ksba_der_builder_get (ksba_der_t d, unsigned char **r_obj, size_t *r_objlen)
+{
+  return _ksba_der_builder_get (d, r_obj, r_objlen);
+}
